# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""Contains the classes system_objects and single_object.

Used to compute system models

"""

from .. import mats
import os
import copy
from .. import solvers
import object


class system_objects:

    """system_objects class

    This class creates a system of thermal objects, establishes contact between
    them and computes the respective thermal processes.

    """

    def __init__(self, number_objects=2, materials=('Cu', 'Cu'),
                 objects_length=(10, 10), amb_temperature=293, dx=0.01, dt=0.1,
                 file_name='data', initial_state=False,
                 boundaries=((2, 0), (3, 0))):
        """Initializes the object.

        amb_temperature: ambient temperature of the whole system
        materials: list of strings of all the used materials present in the
            folder materials
        number_objects: integer for the number of thermal objects
        objects_length: list of the object lengths (spacial steps)
        dx: the space step
        dt: the times step
        file_name: file name where the temperature and heat flux are saved
        boundaries: tuple of two entries that define the boundary condition
            for tempreture. The first corresponds to the thermal obect while
            the second defines the temperature. If 0 the boundary condition is
            insulation
        initial_state: initial state of the materials. True if applied field
            and False is removed field.

        """

        # check the validity of inputs
        cond01 = isinstance(amb_temperature, float)
        cond01 = cond01 or isinstance(amb_temperature, int)
        cond02 = isinstance(materials, tuple)
        cond03 = isinstance(number_objects, int)
        cond04 = isinstance(objects_length, tuple)
        cond05 = isinstance(dx, int) or isinstance(dx, float)
        cond06 = isinstance(dt, int) or isinstance(dt, float)
        cond07 = isinstance(file_name, unicode)
        cond07 = cond07 or isinstance(file_name, str)
        cond08 = isinstance(boundaries, tuple)
        cond09 = isinstance(initial_state, bool)
        condition = cond01 and cond02 and cond03 and cond04 and cond05
        condition = condition and cond06 and cond07 and cond08 and cond09
        if not condition:
            raise ValueError

        # initial definitions
        self.objects = []
        for i in range(number_objects):
            if i not in [l[0] for l in boundaries] or (i, 0) in boundaries:
                heat_save = False
            else:
                heat_save = True

            self.objects.append(object.object(amb_temperature,
                                materials=(materials[i],),
                                borders=(1, objects_length[i]+1),
                                materials_order=(0,), dx=dx, dt=dt,
                                file_name=file_name+'_'+str(i)+'.txt',
                                boundaries=(0, 0), Q=[], Q0=[],
                                initial_state=initial_state,
                                heat_save=heat_save))

        self.contacts = set()
        self.boundaries = boundaries
        self.dt = dt
        self.q1 = 0.
        self.q2 = 0.

        for i in boundaries:
            if i[1] != 0:
                for j in range(len(self.objects[i[0]].temperature)):
                    self.objects[i[0]].temperature[j] = [i[1], i[1]]

    def contactFilter(self, object):
        """Filter self.contacts by thermal object id

        object: thermal object id

        """

        filtered = [x for x in
                    self.contacts if (x[0][0] == object or x[1][0] == object)]
        return set(filtered)

        self.contacts.add(contact)

    def contactAdd(self, contact):
        """Add contact to self.contacts

        contact: thermal contact

        """

        self.contacts.add(contact)

    def contactRemove(self, contact):
        """Remove all contacts from an object

        contact: thermal object id

        """

        removing_contact = None

        for i in range(len(self.contacts)):
            if self.contacts[i][0] == contact:
                removing_contact = self.contacts[i]

        self.contacts.remove(removing_contact)

    def compute(self, timeInterval, writeInterval, solver='implicit_k'):
        """Computes the thermal process

        Computes the system for timeInterval, and writes into the file_name
        file every writeInterval time steps. Four different solvers can be
        used: 'explicit_general', 'explicit_k(x)', 'implicit_general',
        and 'implicit_k(x)'.

        """

        # number of time steps for the given timeInterval
        nt = int(timeInterval / self.dt)

        # number of time steps counting from the last writing process
        nw = 0

        # computes
        for j in range(nt):

            for obj in self.objects:
                obj.Q0 = copy.copy(obj.Q0_ref)

            for contact in self.contacts:
                td1 = self.objects[contact[1][0]].temperature[contact[1][1]][0]
                td2 = self.objects[contact[0][0]].temperature[contact[0][1]][0]
                heat_contact_1 = contact[2] * (td1 - td2)
                heat_contact_2 = contact[2] * (td2 - td1)
                self.objects[contact[0][0]].Q0[contact[0][1]] = heat_contact_1
                self.objects[contact[1][0]].Q0[contact[1][1]] = heat_contact_2

            object_number = -1
            for obj in self.objects:
                object_number = object_number + 1
                obj.time_passed = obj.time_passed + obj.dt

                cond1 = object_number not in [l[0] for l in self.boundaries]
                if cond1 or (object_number, 0) in self.boundaries:

                    # defines the material properties
                    for i in range(1, obj.num_points - 1):
                        if obj.state[i] is True:
                            ind = obj.materials_index[i]
                            obj.rho[i] = obj.materials[ind].rhoa(
                                obj.temperature[i][0])
                            obj.Cp[i] = obj.materials[ind].cpa(
                                obj.temperature[i][0])
                            obj.k[i] = obj.materials[ind].ka(
                                obj.temperature[i][0])
                        if obj.state[i] is False:
                            ind = obj.materials_index[i]
                            obj.rho[i] = obj.materials[ind].rho0(
                                obj.temperature[i][0])
                            obj.Cp[i] = obj.materials[ind].cp0(
                                obj.temperature[i][0])
                            obj.k[i] = obj.materials[ind].k0(
                                obj.temperature[i][0])

                    # SOLVERS

                    # implicit k constant
                    if solver == 'implicit_general':
                        obj.temperature = solvers.implicit_general(obj)

                    # implicit k dependent on x
                    if solver == 'implicit_k(x)':
                        obj.temperature = solvers.implicit_k(obj)

                    # explicit k constant
                    if solver == 'explicit_general':
                        obj.temperature = solvers.explicit_general(obj)

                    # explicit k dependent on x
                    if solver == 'explicit_k(x)':
                        obj.temperature = solvers.explicit_K(obj)

                    # writes the temperature to file_name file ...
                    # if the number of time steps is verified
                    if nw + 1 == writeInterval or j == 0 or j == nt - 1:
                        line = '%f' % obj.time_passed
                        for i in obj.temperature:
                            new_line = ',%f' % i[1]
                            line = line + new_line
                        f = open(obj.file_name, 'a')
                        f.write(line+'\n')
                        f.close()

                else:

                    heat = [p*self.dt*obj.dx for p in obj.Q0 if p is not None]
                    heat = sum(heat)/(len(heat)*obj.dx)

                    if object_number == self.boundaries[0][0]:
                        self.q1 = self.q1 + heat
                        q = self.q1
                    else:
                        self.q2 = self.q2 + heat
                        q = self.q2

                    # writes the temperature to file_name file ...
                    # if the number of time steps is verified
                    if nw + 1 == writeInterval or j == 0 or j == nt - 1:
                        line = '%f' % obj.time_passed
                        for i in obj.temperature:
                            new_line = ',%f' % i[1]
                            line = line + new_line
                        new_line = ',%f' % q
                        line = line + new_line
                        f = open(obj.file_name, 'a')
                        f.write(line+'\n')
                        f.close()

            if nw == writeInterval:
                nw = 0
            else:
                nw = nw + 1


class single_object(object.object):

    """single_object class

    This class solves numerically the heat conduction equation for 1 dimension
    of an active material(s). Three solvers can be used: explicit with
    x-independent k, explicit with x-dependent k, implicit with x-independent
    k, and implicit with x-dependent k. The class has 5 methods: activate for
    the activation of part of the solid, deactivate for the deactivation of
    part of the solid, and compute for solving the equation for a given period
    of time. This class is suited for simulations involving caloric systems
    such as magnetocaloric or electrocaloric systems.

    """

    def __init__(self, amb_temperature, materials=('Cu'), borders=(1, 11),
                 materials_order=(0,), dx=0.01, dt=0.1, file_name='data.txt',
                 boundaries=(0, 0), Q=[], Q0=[], heat_points=(1, -2),
                 initial_state=False, h_left=50000., h_right=50000.):
        """Initializes the object.

        amb_temperature: ambient temperature of the whole system
        materials: list of strings of all the used materials present in the
            folder materials
        borders: list of the points where there is a change of material
        materials_order: list of the materials list indexes that defines the
            material properties given by borders
        dx: the space step
        dt: the times step
        file_name: file name where the temperature and heat flux are saved
        boundaries: list of two entries that define the boundary condition
            for tempreture. If 0 the boundary condition is insulation
        Q: list of 3 entry lists that gives the fixed heat source coeficient.
            The first term is the initial space index where it is applies. The
            second is the final space index where it is applies. The third is
            the value of the coeficient.
        Q0 is a list of 3 entry lists that gives the temperature dependent heat
            source coefficient. The first term is the initial space index where
            it is applies. The second is the final space index where it is
            applies. The third is the value of the coeficient.
        heat_points: list of the space indexes where we want to extract the
            heat flux. Normally, the first term is the heat flux of the hot end
            and the second term is the heat flux of the cold end
        initial_state: initial state of the materials. True if applied field
            and False is removed field.
        h_left: left heat transfer coefficient
        h_right: right heat transfer coefficient

        """

        # check the validity of inputs
        cond01 = isinstance(amb_temperature, float)
        cond01 = cond01 or isinstance(amb_temperature, int)
        cond02 = isinstance(materials, tuple)
        cond03 = isinstance(borders, tuple)
        cond04 = isinstance(materials_order, tuple)
        cond05 = isinstance(dx, int) or isinstance(dx, float)
        cond06 = isinstance(dt, int) or isinstance(dt, float)
        cond07 = isinstance(file_name, unicode)
        cond07 = cond07 or isinstance(file_name, str)
        cond08 = isinstance(boundaries, tuple)
        cond09 = isinstance(heat_points, tuple)
        cond10 = isinstance(initial_state, bool)
        cond11 = isinstance(h_left, int) or isinstance(h_left, float)
        cond12 = isinstance(h_right, int) or isinstance(h_right, float)
        condition = cond01 and cond02 and cond03 and cond04 and cond05
        condition = condition and cond06 and cond07 and cond08 and cond09
        condition = condition and cond10 and cond11 and cond12
        if not condition:
            raise ValueError

        # initial definitions
        self.heat_points = heat_points
        self.borders = borders
        self.materials = range(len(materials))
        self.boundaries = boundaries
        self.amb_temperature = amb_temperature
        self.h_left = h_left
        self.h_right = h_right

        # loads the data for each material
        for i in range(len(materials)):
            tadi = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'tadi.txt'
            tadd = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'tadd.txt'
            cpa = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'cpa.txt'
            cp0 = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'cp0.txt'
            k0 = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'k0.txt'
            ka = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'ka.txt'
            rho0 = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'rho0.txt'
            rhoa = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'rhoa.txt'
            self.materials[i] = mats.calmatpro(
                tadi, tadd, cpa, cp0, k0, ka, rho0, rhoa)

        # defines which are the properties of each material point
        self.materials_index = [None]
        for i in range(len(borders) - 1):
            for j in range(borders[i], borders[i + 1]):
                self.materials_index.append(materials_order[i])
        self.materials_index.append(None)

        # loads the inputs and creates the lists temperature, Cp, rho and k
        self.num_points = borders[-1] + 1
        self.file_name = file_name
        self.dx = dx
        self.dt = dt
        self.temperature = [[amb_temperature, amb_temperature]]
        self.Cp = [None]
        self.rho = [None]
        self.Q = [None]
        self.Q0 = [None]
        self.k = [self.materials[self.materials_index[1]].k0(amb_temperature)]
        for i in range(1, borders[-1]):
            self.temperature.append([amb_temperature, amb_temperature])
            self.rho.append(self.materials[self.materials_index[i]].rho0(
                amb_temperature))
            self.Cp.append(
                self.materials[self.materials_index[i]].cp0(amb_temperature))
            self.k.append(
                self.materials[self.materials_index[i]].k0(amb_temperature))
            self.Q.append(0.)
            self.Q0.append(0.)
        self.temperature.append([amb_temperature, amb_temperature])
        self.rho.append(None)
        self.Cp.append(None)
        self.Q.append(None)
        self.Q0.append(None)
        self.k.append(
            self.materials[self.materials_index[-2]].k0(amb_temperature))

        # creates the power sources
        for power in Q:
            for j in range(power[1], power[2]):
                self.Q[j] = power[0]
        for power in Q0:
            for j in range(power[1], power[2]):
                self.Q0[j] = power[0]

        # initializes the state of each point (True if active and False if ...
        # unactive), the time, heat flux from the hot side and heat flux ...
        # from the cold side
        self.state = [initial_state for i in range(borders[-1] + 1)]
        self.time_passed = 0.
        self.heatLeft = 0.
        self.heatRight = 0.

        line = 'time(s)'
        for i in range(len(self.temperature)):
            line = line + ',T[' + str(i) + '] (K)'
        line = line + ',heat[' + str(self.heat_points[0]) + '](W)' + \
            ',heat[' + str(self.heat_points[1]) + '](J)\n'
        f = open(self.file_name, 'a')
        f.write(line)
        f.close()

    def changeHeatPower(self, Q=[], Q0=[]):
        """Heat power source change

        Changes the coeficients for the heat power sources

        """

        for power in Q:
            for j in range(power[1], power[2]):
                self.Q[j] = power[0]
        for power in Q0:
            for j in range(power[1], power[2]):
                self.Q0[j] = power[0]

    def compute(self, timeInterval, writeInterval, solver='explicit_k(x)',
                modeTemp=False, numFlag=0.5, modeTempPoint=1):
        """Computes the thermal process

        Computes the system for timeInterval, and writes into the file_name
        file every writeInterval time steps. Four different solvers can be
        used: 'explicit_general', 'explicit_k(x)', 'implicit_general',
        and 'implicit_k(x)'. heat_points is a list that defines the points
        where the heat flux are calculated if modeTemp is True the compute
        method stops when the point modeTempPoint changes numFlag relative to
        the initial value

        """

        # number of time steps for the given timeInterval
        nt = int(timeInterval / self.dt)

        # number of time steps counting from the last writing process
        nw = 0

        # saves the reference temperature for the modeTemp
        initModeTemp = self.temperature[modeTempPoint][0]

        # computes
        for j in range(nt):

            # updates the time_passed
            self.time_passed = self.time_passed + self.dt

            # defines the material properties accoring to the state list
            for i in range(1, self.num_points - 1):
                if self.state[i] is True:
                    self.rho[i] = self.materials[self.materials_index[i]].rhoa(
                        self.temperature[i][0])
                    self.Cp[i] = self.materials[self.materials_index[i]].cpa(
                        self.temperature[i][0])
                    self.k[i] = self.materials[self.materials_index[i]].ka(
                        self.temperature[i][0])
                if self.state[i] is False:
                    self.rho[i] = self.materials[self.materials_index[i]].rho0(
                        self.temperature[i][0])
                    self.Cp[i] = self.materials[self.materials_index[i]].cp0(
                        self.temperature[i][0])
                    self.k[i] = self.materials[self.materials_index[i]].k0(
                        self.temperature[i][0])

            # SOLVERS

            # implicit k constant
            if solver == 'implicit_general':
                self.temperature = solvers.implicit_general(self)

            # implicit k dependent on x
            if solver == 'implicit_k(x)':
                self.temperature = solvers.implicit_k(self)

            # explicit k constant
            if solver == 'explicit_general':
                self.temperature = solvers.explicit_general(self)

            # explicit k dependent on x
            if solver == 'explicit_k(x)':
                self.temperature = solvers.explicit_k(self)

            # calculates the heat flux of the defined ...
            # two points during the time step

            self.heatLeft = (-self.dt * self.h_left *
                             (self.temperature[self.heat_points[0]][1] -
                              self.boundaries[0]) + self.heatLeft)
            self.heatRight = (self.dt * self.h_right *
                              (self.temperature[self.heat_points[1]][1] -
                               self.boundaries[1]) + self.heatRight)

            nw = nw + 1

            # stops the simulation if modeTemp is True ...
            # and the stop condition is verified
            if modeTemp is True:
                value = self.temperature[modeTempPoint][0] - initModeTemp
                if abs(value) > numFlag:
                    nw = 0
                    f = open(self.file_name, 'a')
                    f.write(line)
                    f.close()
                    break

            # writes the temperature to file_name file ...
            # if the number of time steps is verified
            if nw == writeInterval or j == 0 or j == nt - 1:
                line = '%f,' % self.time_passed
                for i in self.temperature:
                    new_line = '%f,' % i[1]
                    line = line + new_line
                new_line = '%f,' % self.heatLeft
                line = line + new_line
                new_line = '%f' % self.heatRight
                line = line + new_line + '\n'
                f = open(self.file_name, 'a')
                f.write(line)
                f.close()

            if nw == writeInterval:
                nw = 0



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""heatrapy: Library for simulating heat transfer processes

Computes heat transfer processes in solids for 1-dimensional models.
It includes two models for simulating caloric systems.

"""

from setuptools import setup

description = 'Computes heat transfer processes in solids for 1-dimensional '
description += 'models. It includes two models for simulating caloric '
description += 'systems.'


setup(name='heatrapy',
      version='0.2.0',
      description='Library for simulating heat transfer processes',
      long_description=description,
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 2.7',
          'Topic :: Scientific/Engineering :: Physics',
      ],
      url='https://github.com/danieljosesilva/heatrapy',
      author='Daniel Silva',
      author_email='djsilva99@gmail.com',
      license='MIT',
      packages=['heatrapy'],
      install_requires=[
          'numpy',
      ],
      include_package_data=True,
      zip_safe=False)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""Contains the function solid_active_regenerator.

Used to compute 1-dimensional models for solid state systems involving only one
thermal object and caloric effects.

"""

from .. import objects
from .. import fields
import time
import numpy as np


def solid_active_regenerator(file_name, amb_temperature=293,
                             left_thermalswitch_length=2,
                             right_thermalswitch_length=2, MCM_length=20,
                             right_reservoir_length=3, left_reservoir_length=3,
                             MCM_material=((0.002, 'Gd'),),
                             left_thermalswitch_material='idealTS_hot',
                             right_thermalswitch_material='idealTS_cold',
                             left_reservoir_material='Cu',
                             right_reservoir_material='Cu',
                             freq=.1, dt=.01, dx=0.002, stop_criteria=5e-3,
                             solver='implicit_k(x)', min_cycle_number=30,
                             max_cycle_number=31, field_removal_steps=3,
                             field_applied_steps=1,
                             field_removal_mode='accelerated_right',
                             field_applied_mode='accelerated_left',
                             cycle_points=25, boundaries=(293, 293), note=None,
                             temperature_sensor='default',
                             heat_points='default', mode='refrigerator',
                             version=None, resting_time_hot='default',
                             resting_time_cold='default',
                             starting_field='applied',
                             type_study='fixed_temperature_span',
                             h_left=50000., h_right=50000.,
                             mod_freq='default'):
    """solid_active_regenerator class

    computes the thermal processes for 1-dimensional fully solid state
    ferroic-based systems. The active regenerative processes can be used with
    the several allowed modes for application and removal of fields. Cascades
    of materials can also be computed.

    file_name: file name where the temperature and heat flux are saved
    amb_temperature: ambient temperature of the whole system
    left_thermalswitch_length: length of the left thermal switch
    right_thermalswitch_length: length of the right thermal switch
    left_thermalswitch_material: string for the material of the
        left thermal switch
    right_thermalswitch_material: string for the material of the
        right thermal switch
    MCM_length: length of the caloric material
    MCM_material: string for the material of the caloric material
    left_reservoir_length: length of the left reservoir
    right_reservoir_length: length of the right reservoir
    left_reservoir_material: string for the material of the left reservoir
    right_reservoir_material: string for the material of the right reservoir
    freq: operating frequency
    dt: times step
    dx: space step
    stop_criteria: error threshold to stop the simulation
    solver: the solver
    min_cycle_number: minimum number of cycles that has to be computed
    max_cycle_number: maximum number of cycles that has to be computed
    field_removal_steps: number of steps during the field removal
    field_applied_steps: number of steps during the application of field
    field_removal_mode: mode of field removal
        modes can be constant_right, constant_left, accelerated_right,
        accelerated_left, decelerated_right, and decelerated_left
    field_applied_mode is the mode of the application of field
        modes can be constant_right, constant_left, accelerated_right,
        accelerated_left, decelerated_right, and decelerated_left
    cycle_points: number of points recorded for each position for each cycle
    boundaries: list with the boundary conditions
    temperature_sensor: list of two space indexes used to determine the
        temperature span at the end of the simulation. The first term is
        the sensor at the hot end and the second at the cold end
    heat_points: list of two space indexes used to determine the heat
        flux for the hot end (first term) and cold end (second term)
    mode: mode used for the power calculations (e.g. COP) performed at
        the end of the simulation
    version: heatrapy version (default is None)
    type_study: 'no_load' or 'fixed_temperature_span'
    h_left: left heat transfer coefficient
    h_right: right heat transfer coefficient
    mod_freq: if not 'default', allows to modulate the frequency according
        to a specific temperature. tuple with first element as file_name,
        and second the sensor point.

    """

    # check the validity of inputs

    cond01 = isinstance(file_name, unicode)
    cond01 = cond01 or isinstance(file_name, str)
    cond02 = isinstance(amb_temperature, float)
    cond02 = cond01 or isinstance(amb_temperature, int)
    cond03 = isinstance(left_thermalswitch_length, int)
    cond04 = isinstance(right_thermalswitch_length, int)
    cond05 = isinstance(MCM_length, int)
    cond06 = isinstance(left_reservoir_length, int)
    cond07 = isinstance(right_reservoir_length, int)
    cond08 = isinstance(MCM_material, tuple)
    cond09 = isinstance(left_thermalswitch_material, str)
    cond09 = cond09 or isinstance(left_thermalswitch_material, unicode)
    cond10 = isinstance(left_thermalswitch_material, str)
    cond10 = cond10 or isinstance(left_thermalswitch_material, unicode)
    cond11 = isinstance(left_reservoir_material, str)
    cond11 = cond11 or isinstance(left_reservoir_material, unicode)
    cond12 = isinstance(right_reservoir_material, str)
    cond12 = cond12 or isinstance(right_reservoir_material, unicode)
    cond13 = isinstance(freq, int) or isinstance(freq, float)
    cond14 = isinstance(dx, int) or isinstance(dx, float)
    cond15 = isinstance(dt, int) or isinstance(dt, float)
    cond16 = isinstance(stop_criteria, int) or isinstance(stop_criteria, float)
    cond17 = isinstance(solver, unicode) or isinstance(solver, str)
    cond18 = isinstance(min_cycle_number, int)
    cond19 = isinstance(max_cycle_number, int)
    cond20 = isinstance(field_removal_steps, int)
    cond21 = isinstance(field_applied_steps, int)
    allowed_modes = ['constant_right', 'constant_left', 'accelerated_right',
                     'accelerated_left', 'decelerated_right',
                     'decelerated_left']
    cond22 = field_removal_mode in allowed_modes
    cond23 = field_applied_mode in allowed_modes
    cond24 = isinstance(cycle_points, int)
    cond25 = isinstance(boundaries, tuple)
    cond26 = mode == 'refrigerator' or mode == 'heat_pump'
    cond27 = starting_field == 'applied' or starting_field == 'removal'
    cond28 = type_study == 'no_load' or type_study == 'fixed_temperature_span'
    cond29 = isinstance(h_left, int) or isinstance(h_left, float)
    cond30 = isinstance(h_left, int) or isinstance(h_right, float)
    condition = cond01 and cond02 and cond03 and cond04 and cond05
    condition = condition and cond06 and cond07 and cond08 and cond09
    condition = condition and cond10 and cond11 and cond12 and cond13
    condition = condition and cond14 and cond15 and cond16 and cond17
    condition = condition and cond18 and cond19 and cond20 and cond21
    condition = condition and cond22 and cond23 and cond24 and cond25
    condition = condition and cond26 and cond27 and cond28 and cond29
    condition = condition and cond30
    if not condition:
        raise ValueError

    # restingTimes definition
    if resting_time_hot == 'default':
        resting_time_hot = 0.

    if resting_time_cold == 'default':
        resting_time_cold = 0.

    # information for the log file
    print ''
    print ''
    print '######################################################'
    print ''
    print '------------------------------------------------------'
    print file_name
    print '------------------------------------------------------'
    print ''
    print 'heatconpy version:', version
    print 'Module: solid_active_regenerator'
    if note is not None:
        print ''
        print 'Note:', note
    print ''
    print 'Mode:', mode
    print 'System:', left_reservoir_material + '/' + \
        left_thermalswitch_material + '/MCM material/' + \
        right_thermalswitch_material + '/' + right_reservoir_material
    print 'Dimensions (m):', str(dx * left_reservoir_length) + '/' + \
        str(dx * left_thermalswitch_length) + '/' + str(dx * MCM_length) + \
        '/' + str(dx * right_thermalswitch_length) + '/' + \
        str(dx * right_reservoir_length)
    print 'Number of points:', str(left_reservoir_length) + '/' + \
        str(left_thermalswitch_length) + '/' + str(MCM_length) + '/' + \
        str(right_thermalswitch_length) + '/' + str(right_reservoir_length)
    print 'MCM material:', MCM_material
    print 'Left heat transfer coefficient (W /(m2 K)):', h_left
    print 'Right heat transfer coefficient (W /(m2 K)):', h_right
    print 'dx (m):', dx
    print 'dt (s):', dt
    print 'Frequency (Hz):', freq
    print 'Hot resting time ratio:', resting_time_hot
    print 'Cold resting time ratio:', resting_time_cold
    print 'Solver:', solver
    print 'Frequency modulation:', mod_freq
    print 'Applied field mode:', field_applied_mode
    print 'Applied field steps:', field_applied_steps
    print 'Field removal mode:', field_removal_mode
    print 'Field removal steps:', field_removal_steps
    print 'Starting Field:', starting_field
    print 'Boundaries:', boundaries
    print 'Ambient temperature (K):', amb_temperature
    print 'Stop criteria:', stop_criteria
    print 'Time:', time.ctime()
    print ''

    # used to calculate the simulation time at the end
    start_time = time.time()

    if heat_points == 'default':
        if left_reservoir_length - 3 > 0:
            leftHeatSensor = left_reservoir_length - 3
        else:
            leftHeatSensor = 2
        if -right_reservoir_length + 3 < 0:
            rightHeatSensor = -right_reservoir_length + 3
        else:
            rightHeatSensor = -2
    else:
        leftHeatSensor = heat_points[0]
        rightHeatSensor = heat_points[1]

    if starting_field != 'applied':
        initial_state = True
    else:
        initial_state = False

    # initializes the object for the simulation
    materials = (left_reservoir_material, left_thermalswitch_material,
                 MCM_material[0][1], right_thermalswitch_material,
                 right_reservoir_material)

    lRl = left_reservoir_length
    rRl = right_reservoir_length
    ltsl = left_thermalswitch_length
    rtsl = right_thermalswitch_length
    borders = (1, lRl + 1,
               lRl + ltsl + 1,
               lRl + ltsl + MCM_length + 1,
               lRl + ltsl + MCM_length + rtsl + 1,
               lRl + ltsl + MCM_length + rtsl + rRl + 1)
    heat_points = (leftHeatSensor, rightHeatSensor)

    a = objects.single_object(amb_temperature, dx=dx, dt=dt,
                              file_name=file_name, materials=materials,
                              borders=borders, materials_order=(0, 1, 2, 3, 4),
                              boundaries=boundaries, heat_points=heat_points,
                              initial_state=initial_state, h_left=50000.,
                              h_right=50000.)

    # defines the material cascade
    k = left_reservoir_length + left_thermalswitch_length
    for i in range(len(MCM_material)):
        from .. import mats
        import os
        tadi = os.path.dirname(os.path.realpath(__file__)) + \
            '/../database/' + MCM_material[i][1] + '/' + 'tadi.txt'
        tadd = os.path.dirname(os.path.realpath(__file__)) + \
            '/../database/' + MCM_material[i][1] + '/' + 'tadd.txt'
        cpa = os.path.dirname(os.path.realpath(__file__)) + \
            '/../database/' + MCM_material[i][1] + '/' + 'cpa.txt'
        cp0 = os.path.dirname(os.path.realpath(__file__)) + \
            '/../database/' + MCM_material[i][1] + '/' + 'cp0.txt'
        k0 = os.path.dirname(os.path.realpath(__file__)) + \
            '/../database/' + MCM_material[i][1] + '/' + 'k0.txt'
        ka = os.path.dirname(os.path.realpath(__file__)) + \
            '/../database/' + MCM_material[i][1] + '/' + 'ka.txt'
        rho0 = os.path.dirname(os.path.realpath(__file__)) + \
            '/../database/' + MCM_material[i][1] + '/' + 'rho0.txt'
        rhoa = os.path.dirname(os.path.realpath(__file__)) + \
            '/../database/' + MCM_material[i][1] + '/' + 'rhoa.txt'
        a.materials.append(mats.calmatpro(tadi, tadd, cpa, cp0, k0, ka,
                                          rho0, rhoa))
        for j in range(k, k+int(MCM_material[i][0]/dx+1)):
            a.materials_index[j] = len(a.materials)-1
        k = k+int(MCM_material[i][0]/dx)

    # defines some variable for the cycles
    if temperature_sensor == 'default':
        righttemperature_sensor = -(right_reservoir_length / 2)
        lefttemperature_sensor = (left_reservoir_length / 2)
    else:
        righttemperature_sensor = temperature_sensor[1]
        lefttemperature_sensor = temperature_sensor[0]

    value1 = amb_temperature
    value2 = amb_temperature
    i = 0
    period = (1. / freq)
    stop_criteria2 = 0.
    maximumPower = 0.
    maximumWorkingPower = 0.

    # cycle simulation
    while ((abs((value1 - value2) / value2) > stop_criteria or
            i < min_cycle_number or
            abs((value1 - value2) / value2) > stop_criteria2) and
            i < max_cycle_number):

        stop_criteria2 = abs((value1 - value2) / value2)
        heatLeft = a.heatLeft
        heatRight = a.heatRight

        if mod_freq != 'default':
            temperature_sensor = a.temperature[mod_freq[1]][0]
            input = open(mod_freq[0], 'r')
            s = input.readlines()
            xMod = []
            yMod = []
            for line in s:
                pair = line.split(',')
                xMod.append(float(pair[0]))
                yMod.append(float(pair[1]))
            input.close()
            freq = np.interp(temperature_sensor, xMod, yMod)
            period = (1. / freq)

        if starting_field == 'applied':

            # APPLICATION OF FIELD AND COMPUTATION

            com = ['constant_left', 'accelerated_left', 'decelerated_left']
            if field_applied_mode in com:
                step_range = range(field_applied_steps - 1, -1, -1)
            else:
                step_range = range(field_applied_steps)

            for j in step_range:

                first = (left_reservoir_length + j *
                         (left_thermalswitch_length + MCM_length +
                          right_thermalswitch_length) /
                         field_applied_steps + 1)
                second = (left_reservoir_length + (j + 1) *
                          (left_thermalswitch_length + MCM_length +
                           right_thermalswitch_length) /
                          field_applied_steps + 1)
                a.activate(first, second)
                delta_t = fields.operating_mode(field_applied_mode,
                                                resting_time_cold,
                                                resting_time_hot,
                                                field_applied_steps, freq,
                                                j)
                a.compute(delta_t, int(1 / (freq * dt * cycle_points)),
                          solver=solver)

            a.compute(resting_time_hot * period,
                      int(1 / (freq * dt * cycle_points)),
                      solver=solver)

            # REMOVAL OF FIELD AND COMPUTATION

            com = ['constant_left', 'accelerated_left', 'decelerated_left']
            if field_applied_mode in com:
                step_range = range(field_removal_steps - 1, -1, -1)
            else:
                step_range = range(field_removal_steps)

            for j in step_range:
                first = (left_reservoir_length + j *
                         (left_thermalswitch_length + MCM_length +
                          right_thermalswitch_length) /
                         field_removal_steps + 1)
                second = (left_reservoir_length + (j + 1) *
                          (left_thermalswitch_length + MCM_length +
                           right_thermalswitch_length) /
                          field_removal_steps + 1)
                a.deactivate(first, second)
                delta_t = fields.operating_mode(field_removal_mode,
                                                resting_time_cold,
                                                resting_time_hot,
                                                field_removal_steps, freq,
                                                j)
                a.compute(delta_t, int(1 / (freq * dt * cycle_points)),
                          solver=solver)

            time_interval = resting_time_cold * period
            write_interval = int(1 / (freq * dt * cycle_points))
            a.compute(time_interval, write_interval, solver=solver)

        if starting_field == 'removal':

            # REMOVAL OF FIELD AND COMPUTATION

            com = ['constant_left', 'accelerated_left', 'decelerated_left']
            if field_applied_mode in com:
                step_range = range(field_removal_steps - 1, -1, -1)
            else:
                step_range = range(field_removal_steps)

            for j in step_range:
                first = (left_reservoir_length + j *
                         (left_thermalswitch_length + MCM_length +
                          right_thermalswitch_length) /
                         field_removal_steps + 1)
                second = (left_reservoir_length + (j + 1) *
                          (left_thermalswitch_length + MCM_length +
                           right_thermalswitch_length) /
                          field_removal_steps + 1)
                a.deactivate(first, second)
                delta_t = fields.operating_mode(field_removal_mode,
                                                resting_time_cold,
                                                resting_time_hot,
                                                field_removal_steps, freq,
                                                j)
                a.compute(delta_t, int(1 / (freq * dt * cycle_points)),
                          solver=solver)

            time_interval = resting_time_cold * period
            write_interval = int(1 / (freq * dt * cycle_points))
            a.compute(time_interval, write_interval, solver=solver)

        # APPLICATION OF FIELD AND COMPUTATION

            com = ['constant_left', 'accelerated_left', 'decelerated_left']
            if field_applied_mode in com:
                step_range = range(field_applied_steps - 1, -1, -1)
            else:
                step_range = range(field_applied_steps)

            for j in step_range:
                first = (left_reservoir_length + j *
                         (left_thermalswitch_length + MCM_length +
                          right_thermalswitch_length) /
                         field_applied_steps + 1)
                second = (left_reservoir_length + (j + 1) *
                          (left_thermalswitch_length + MCM_length +
                           right_thermalswitch_length) /
                          field_applied_steps + 1)
                a.activate(first, second)
                delta_t = fields.operating_mode(field_applied_mode,
                                                resting_time_cold,
                                                resting_time_hot,
                                                field_applied_steps, freq,
                                                j)
                a.compute(delta_t, int(1 / (freq * dt * cycle_points)),
                          solver=solver)

            a.compute(resting_time_hot * period,
                      int(1 / (freq * dt * cycle_points)),
                      solver=solver)

        # updates the error values and prints information in log file
        value1 = value2
        value2 = a.temperature[righttemperature_sensor][1]
        i = 1 + i

    # calculates the final temperatures
    before_final_cycle_time = a.timePassed - 1. / freq
    time_accumulated = 0.
    file_temperature = open(file_name)
    lines = file_temperature.readlines()
    final_temperature_right = 0.
    o = 1
    while float(lines[o].split(',')[0]) < before_final_cycle_time:
        lines.pop(0)
        o = o + 1
    for o in range(1, len(lines)):
        time_before = float(lines[o - 1].split(',')[0])
        time_now = float(lines[o].split(',')[0])
        sensor = righttemperature_sensor - 2
        final_temperature_right = ((time_now - time_before) *
                                   float(lines[o].split(',')[sensor]) +
                                   final_temperature_right)
        time_accumulated = time_accumulated + (time_now - time_before)
    final_temperature_right = final_temperature_right / time_accumulated
    file_temperature.close()

    before_final_cycle_time = a.timePassed - 1. / freq
    time_accumulated = 0.
    file_temperature = open(file_name)
    lines = file_temperature.readlines()
    final_temperature_left = 0.
    o = 1
    while float(lines[o].split(',')[0]) < before_final_cycle_time:
        lines.pop(0)
        o = o + 1
    for o in range(1, len(lines)):
        time_before = float(lines[o - 1].split(',')[0])
        time_now = float(lines[o].split(',')[0])
        sensor = lefttemperature_sensor + 1
        final_temperature_left = ((time_now - time_before) *
                                  float(lines[o].split(',')[sensor]) +
                                  final_temperature_left)
        time_accumulated = time_accumulated + (time_now - time_before)
    final_temperature_left = final_temperature_left / time_accumulated
    file_temperature.close()

    # prints more information in log file
    endTime = time.time()
    simulationTime = endTime - start_time
    hours = int(simulationTime / 3600)
    minutes = int((simulationTime - hours * 3600) / 60)
    seconds = int(simulationTime - hours * 3600 - (minutes * 60))
    hours = '%02d' % hours
    minutes = '%02d' % minutes
    seconds = '%02d' % seconds

    print '------------------------------------------------------'
    print ''
    print 'Number of cycles:', i
    print 'Final cycle error:', abs((value1 - value2) / value2)
    if type_study == 'fixed_temperature_span':
        heating_power = -(a.heatLeft - heatLeft) / period
        cooling_power = -(a.heatRight - heatRight) / period
        working_power = heating_power - cooling_power
        print 'Heating power (W):', heating_power
        print 'Cooling power (W):', cooling_power
        print 'Working power (W):', working_power
        if mode == 'refrigerator':
            print 'COP:', cooling_power / working_power
        if mode == 'heat_pump':
            print 'COP:', heating_power / working_power
    else:
        temperature_span = (-final_temperature_right +
                            final_temperature_left)
        if mode == 'refrigerator':
            print 'No load temperature span (K):', temperature_span
        if mode == 'heat_pump':
            print 'No load temperature span (K):', -temperature_span
    print 'Final time (s):', a.timePassed
    print 'Simulation duration:', hours + ':' + minutes + ':' + seconds
    print ''
    print '------------------------------------------------------'
    print ''
    print ''
    print ''
